#include "DeerRender/UIEngine/DockPanelObject.h"
#include "DeerRender/UIEngine/UIEngine.h"
#include "DeerRender/UIEngine/UIEngine_ErrorHandle.h"

#include "angelscript.h"
#include <string>

#include "imgui.h"

namespace Deer {
    UIEngine::DockPanelObject::DockPanelObject(asITypeInfo* _type)
        : type (_type), isValid(false) {
        
        // Constructor
        // "type@ type()" 
        std::string callString(type->GetName());
        callString += " @";
        callString += type->GetName();
        callString += "()";

        asIScriptFunction* factory = type->GetFactoryByDecl(callString.c_str());
        
        AS_CHECK_ADDITIONAL_INFO(
            scriptContext->Prepare(factory),
            type->GetName()
        );
        
        AS_CHECK_ADDITIONAL_INFO(
            scriptContext->Execute(),
            type->GetName()
        );

        // Return value contains the ref to a asIScriptObject in the location provided
        object = *(asIScriptObject**)scriptContext->GetAddressOfReturnValue();
        if (!object){
            DEER_UI_ENGINE_ERROR("Could not create object", _type->GetName());
            return;
        }
        object->AddRef();
        
        renderFunction = type->GetMethodByDecl("void onRender()");
        if (!renderFunction) {
            DEER_UI_ENGINE_ERROR("Could not load void onRender() from {0}", _type->GetName());
            return;
        }

        isValid = true;
    }

    UIEngine::DockPanelObject::~DockPanelObject() {
        if (object)
            object->Release();
    }

    void UIEngine::DockPanelObject::executeRender() {
        ImGui::Begin(type->GetName());

        if (!isValid) {
            ImGui::TextColored(ImVec4(1, 0.3f, 0.3f, 1), "There was an error, please check the logs or reload the interface");
            ImGui::End();
            return;
        }
        
        AS_CHECK_ADDITIONAL_INFO(
            scriptContext->Prepare(renderFunction),
            type->GetName()
        );

        AS_CHECK_ADDITIONAL_INFO(
            scriptContext->SetObject(object),
            type->GetName()
        );

        AS_CHECK_ADDITIONAL_INFO(
            scriptContext->Execute(),
            type->GetName()
        );

        ImGui::End();
    }

    UIEngine::DockPanelObject::DockPanelObject(DockPanelObject&& other) noexcept 
        : isValid(other.isValid), renderFunction(other.renderFunction), type(other.type), object(other.object) {
        
        other.isValid = false;
        other.renderFunction = nullptr;
        other.type = nullptr;
        other.object = nullptr;
    }

    UIEngine::DockPanelObject& UIEngine::DockPanelObject::operator=(UIEngine::DockPanelObject&& other) noexcept {
        if (this != &other) {
            isValid = other.isValid;
            renderFunction = other.renderFunction;
            type = other.type;
            object = other.object;

            other.isValid = false;
            other.renderFunction = nullptr;
            other.type = nullptr;
            other.object = nullptr;
        }

        return *this;
    }
}